#include "stdafx.h"
#include "HDUSTest.h"
#include "InfoPanel.h"


#define INFO_PANEL_WINDOW_CLASS TEXT("HDUSTest Info Panel")

#define TAB_MARGIN 4




HINSTANCE CInfoPanel::m_hinst=NULL;


bool CInfoPanel::Initialize(HINSTANCE hinst)
{
	WNDCLASS wc;

	m_hinst=hinst;
	wc.style=0;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=NULL;
	wc.lpszMenuName=NULL;
	wc.lpszClassName=INFO_PANEL_WINDOW_CLASS;
	return RegisterClass(&wc)!=0;
}


CInfoPanel::CInfoPanel()
{
	LOGFONT lf;

	m_WindowPosition.Left=120;
	m_WindowPosition.Top=120;
	m_WindowPosition.Width=200;
	m_WindowPosition.Height=240;
	m_NumWindows=0;
	GetObject(GetStockObject(DEFAULT_GUI_FONT),sizeof(LOGFONT),&lf);
	m_hfont=CreateFontIndirect(&lf);
	m_crBackColor=GetSysColor(COLOR_3DFACE);
	m_crTabBackColor=GetSysColor(COLOR_3DFACE);
	m_crTabTextColor=GetSysColor(COLOR_WINDOWTEXT);
	m_crTabBorderColor=GetSysColor(COLOR_3DDKSHADOW);
	m_crCurTabBackColor=GetSysColor(COLOR_3DHIGHLIGHT);
	m_crCurTabTextColor=GetSysColor(COLOR_WINDOWTEXT);
	m_crCurTabBorderColor=GetSysColor(COLOR_3DDKSHADOW);
	m_TabHeight=abs(lf.lfHeight)+TAB_MARGIN*2;
	m_TabWidth=8+TAB_MARGIN*2;
	m_CurTab=-1;
	m_pEventHandler=NULL;
}


CInfoPanel::~CInfoPanel()
{
	for (int i=0;i<m_NumWindows;i++)
		delete m_pWindowList[i];
	DeleteObject(m_hfont);
}


bool CInfoPanel::Create(HWND hwndParent,DWORD Style,DWORD ExStyle)
{
	return CreateBasicWindow(hwndParent,Style,ExStyle,0,INFO_PANEL_WINDOW_CLASS,TEXT("pl"),m_hinst);
}


bool CInfoPanel::AddWindow(CBasicWindow *pWindow,LPCTSTR pszTitle)
{
	if (m_NumWindows==MAX_WINDOWS)
		return false;
	m_pWindowList[m_NumWindows]=new CWindowInfo(pWindow,pszTitle);
	m_NumWindows++;
	if (m_hwnd!=NULL) {
		CalcTabWidth();
		Invalidate();
	}
	return true;
}


bool CInfoPanel::SetCurTab(int Index)
{
	if (Index<-1 || Index>=m_NumWindows)
		return false;
	if (m_CurTab!=Index) {
		if (m_CurTab>=0)
			m_pWindowList[m_CurTab]->m_pWindow->SetVisible(false);
		if (Index>=0) {
			RECT rc;

			GetClientRect(m_hwnd,&rc);
			m_pWindowList[Index]->m_pWindow->SetPosition(0,m_TabHeight,rc.right,rc.bottom-m_TabHeight);
			m_pWindowList[Index]->m_pWindow->SetVisible(true);
			SetFocus(m_pWindowList[Index]->m_pWindow->GetHandle());
		}
		m_CurTab=Index;
		Invalidate();
	}
	return true;
}


void CInfoPanel::SetEventHandler(CInfoPanelEventHandler *pHandler)
{
	m_pEventHandler=pHandler;
}


void CInfoPanel::SetBackColor(COLORREF crBack)
{
	m_crBackColor=crBack;
	if (m_hwnd!=NULL)
		Invalidate();
}


void CInfoPanel::SetTabColors(COLORREF crBack,COLORREF crText,COLORREF crBorder)
{
	m_crTabBackColor=crBack;
	m_crTabTextColor=crText;
	m_crTabBorderColor=crBorder;
	if (m_hwnd!=NULL)
		Invalidate();
}


void CInfoPanel::SetCurTabColors(COLORREF crBack,COLORREF crText,COLORREF crBorder)
{
	m_crCurTabBackColor=crBack;
	m_crCurTabTextColor=crText;
	m_crCurTabBorderColor=crBorder;
	if (m_hwnd!=NULL)
		Invalidate();
}


CInfoPanel *CInfoPanel::GetThis(HWND hwnd)
{
	return reinterpret_cast<CInfoPanel*>(GetWindowLongPtr(hwnd,GWLP_USERDATA));
}


LRESULT CALLBACK CInfoPanel::WndProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			CInfoPanel *pThis=static_cast<CInfoPanel*>(
					reinterpret_cast<LPCREATESTRUCT>(lParam)->lpCreateParams);

			SetWindowLongPtr(hwnd,GWLP_USERDATA,reinterpret_cast<LONG_PTR>(pThis));
			pThis->m_hwnd=hwnd;
			pThis->CalcTabWidth();
		}
		return 0;

	case WM_PAINT:
		{
			CInfoPanel *pThis=GetThis(hwnd);
			PAINTSTRUCT ps;

			BeginPaint(hwnd,&ps);
			if (ps.rcPaint.top<pThis->m_TabHeight) {
				COLORREF crOldTextColor;
				int OldBkMode;
				HFONT hfontOld;
				int i;
				RECT rc;
				HBRUSH hbr,hbrOld;
				HPEN hpen,hpenOld;

				crOldTextColor=GetTextColor(ps.hdc);
				OldBkMode=SetBkMode(ps.hdc,TRANSPARENT);
				hfontOld=SelectFont(ps.hdc,pThis->m_hfont);
				rc.left=0;
				rc.top=0;
				rc.right=pThis->m_TabWidth;
				rc.bottom=pThis->m_TabHeight;
				for (i=0;i<pThis->m_NumWindows;i++) {
					COLORREF crBack,crText,crBorder;
					RECT rcText;

					if (i==pThis->m_CurTab) {
						crBack=pThis->m_crCurTabBackColor;
						crText=pThis->m_crCurTabTextColor;
						crBorder=pThis->m_crCurTabBorderColor;
					} else {
						crBack=pThis->m_crTabBackColor;
						crText=pThis->m_crTabTextColor;
						crBorder=pThis->m_crTabBorderColor;
					}
					hbr=CreateSolidBrush(crBack);
					hbrOld=SelectBrush(ps.hdc,hbr);
					hpen=CreatePen(PS_SOLID,1,crBorder);
					hpenOld=SelectPen(ps.hdc,hpen);
					Rectangle(ps.hdc,rc.left,rc.top,rc.right,
									i==pThis->m_CurTab?rc.bottom+1:rc.bottom);
					SetTextColor(ps.hdc,crText);
					rcText.left=rc.left+TAB_MARGIN;
					rcText.top=rc.top+TAB_MARGIN;
					rcText.right=rc.right-TAB_MARGIN;
					rcText.bottom=rc.bottom-TAB_MARGIN;
					DrawText(ps.hdc,pThis->m_pWindowList[i]->m_pszTitle,-1,&rcText,
						DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
					SelectPen(ps.hdc,hpenOld);
					DeleteObject(hpen);
					SelectBrush(ps.hdc,hbrOld);
					DeleteObject(hbr);
					rc.left=rc.right;
					rc.right=rc.left+pThis->m_TabWidth;
				}
				SelectFont(ps.hdc,hfontOld);
				SetBkMode(ps.hdc,OldBkMode);
				SetTextColor(ps.hdc,crOldTextColor);
				if (ps.rcPaint.right>pThis->m_NumWindows*pThis->m_TabWidth) {
					rc.left=max(ps.rcPaint.left,(long)(pThis->m_NumWindows*pThis->m_TabWidth));
					rc.top=ps.rcPaint.top;
					rc.right=ps.rcPaint.right;
					rc.bottom=min(ps.rcPaint.bottom,(long)pThis->m_TabHeight-1);
					hbr=CreateSolidBrush(pThis->m_crBackColor);
					FillRect(ps.hdc,&rc,hbr);
					DeleteObject(hbr);
					hpen=CreatePen(PS_SOLID,1,pThis->m_crTabBorderColor);
					hpenOld=SelectPen(ps.hdc,hpen);
					MoveToEx(ps.hdc,rc.left,rc.bottom,NULL);
					LineTo(ps.hdc,rc.right,rc.bottom);
					SelectPen(ps.hdc,hpenOld);
					DeleteObject(hpen);
				}
			}
			EndPaint(hwnd,&ps);
		}
		return 0;

	case WM_SIZE:
		{
			CInfoPanel *pThis=GetThis(hwnd);

			if (pThis->m_CurTab>=0) {
				pThis->m_pWindowList[pThis->m_CurTab]->m_pWindow->SetPosition(0,pThis->m_TabHeight,
							LOWORD(lParam),HIWORD(lParam)-pThis->m_TabHeight);
			}
		}
		return 0;

	case WM_GETMINMAXINFO:
		{
			MINMAXINFO *pmmi=reinterpret_cast<MINMAXINFO*>(lParam);

			pmmi->ptMinTrackSize.x=64;
			pmmi->ptMinTrackSize.y=64;
		}
		return 0;

	case WM_NCHITTEST:
		if (DefWindowProc(hwnd,uMsg,wParam,lParam)==HTCLIENT) {
			CInfoPanel *pThis=GetThis(hwnd);
			POINT pt;

			pt.x=GET_X_LPARAM(lParam);
			pt.y=GET_Y_LPARAM(lParam);
			ScreenToClient(hwnd,&pt);
			if (pt.x<0 || pt.x>=pThis->m_NumWindows*pThis->m_TabWidth)
				return HTCAPTION;
		}
		break;

	case WM_LBUTTONDOWN:
		{
			CInfoPanel *pThis=GetThis(hwnd);
			int x=GET_X_LPARAM(lParam),y=GET_Y_LPARAM(lParam);

			if (y>=0 && y<pThis->m_TabHeight) {
				int Index=x/pThis->m_TabWidth;

				if (Index>=0 && Index<pThis->m_NumWindows
						&& Index!=pThis->m_CurTab) {
					pThis->SetCurTab(Index);
					if (pThis->m_pEventHandler!=NULL)
						pThis->m_pEventHandler->OnSelChange();
				}
			}
		}
		return 0;

	case WM_NCRBUTTONDOWN:
		{
			CInfoPanel *pThis=GetThis(hwnd);

			if (pThis->m_pEventHandler!=NULL) {
				POINT pt;
				RECT rc;

				pt.x=GET_X_LPARAM(lParam);
				pt.y=GET_Y_LPARAM(lParam);
				ScreenToClient(hwnd,&pt);
				GetClientRect(hwnd,&rc);
				if (PtInRect(&rc,pt)) {
					pThis->m_pEventHandler->OnRButtonDown();
					return 0;
				}
			}
		}
		break;

	case WM_KEYDOWN:
		{
			CInfoPanel *pThis=GetThis(hwnd);

			if (pThis->m_pEventHandler!=NULL
					&& pThis->m_pEventHandler->OnKeyDown(wParam,lParam))
				return 0;
		}
		break;

	case WM_MOVING:
		{
			CInfoPanel *pThis=GetThis(hwnd);

			if (pThis->m_pEventHandler!=NULL
					&& pThis->m_pEventHandler->OnMoving(
											reinterpret_cast<LPRECT>(lParam)))
				return TRUE;
		}
		break;

	case WM_ENTERSIZEMOVE:
		{
			CInfoPanel *pThis=GetThis(hwnd);

			if (pThis->m_pEventHandler!=NULL
					&& pThis->m_pEventHandler->OnEnterSizeMove())
				return 0;
		}
		break;

	case WM_CLOSE:
		{
			CInfoPanel *pThis=GetThis(hwnd);

			if (pThis->m_pEventHandler!=NULL
					&& !pThis->m_pEventHandler->OnClose())
				return 0;
		}
		break;

	case WM_DESTROY:
		{
			CInfoPanel *pThis=GetThis(hwnd);

			pThis->OnDestroy();
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}


void CInfoPanel::CalcTabWidth()
{
	HDC hdc;
	HFONT hfontOld;
	int MaxWidth;
	SIZE sz;

	hdc=GetDC(m_hwnd);
	hfontOld=SelectFont(hdc,m_hfont);
	MaxWidth=0;
	for (int i=0;i<m_NumWindows;i++) {
		GetTextExtentPoint32(hdc,m_pWindowList[i]->m_pszTitle,
									lstrlen(m_pWindowList[i]->m_pszTitle),&sz);
		if (sz.cx>MaxWidth)
			MaxWidth=sz.cx;
	}
	SelectFont(hdc,hfontOld);
	ReleaseDC(m_hwnd,hdc);
	m_TabWidth=MaxWidth+TAB_MARGIN*2;
}




CInfoPanel::CWindowInfo::CWindowInfo(CBasicWindow *pWindow,LPCTSTR pszTitle)
{
	m_pWindow=pWindow;
	m_pszTitle=DuplicateString(pszTitle);
}


CInfoPanel::CWindowInfo::~CWindowInfo()
{
	delete [] m_pszTitle;
}
